"""The intro screen for the game"""

import random
import math
import pygame


import serge.actor
import serge.visual
import serge.sound
import serge.events
import serge.common
import serge.blocks.utils
import serge.blocks.visualblocks
import serge.blocks.behaviours
import serge.blocks.actors

if serge.common.PYMUNK_OK:
    import pymunk
    import serge.physical

from theme import G, theme
import common 

class IntroScreen(serge.blocks.actors.ScreenActor):
    """The logic for the intro screen"""
    
    def __init__(self, options):
        """Initialise the screen"""
        super(IntroScreen, self).__init__('item', 'intro-screen')
        self.options = options

    def addedToWorld(self, world):
        """The screen was added to the world"""
        super(IntroScreen, self).addedToWorld(world)
        #
        # Logo
        L = theme.getTheme('intro-screen').getProperty
        # logo = serge.blocks.utils.addSpriteActorToWorld(world, 'logo', 'logo', 'small-icon', 'foreground',
        #     center_position=L('logo-position'))
        # #
        # # Title
        # logo = serge.blocks.utils.addVisualActorToWorld(world, 'title', 'title',
        #     serge.visual.Text(L('title'), L('title-colour'), font_size=L('title-font-size')),
        #     'foreground',
        #     center_position=L('title-position'))
        #
        # Intro text
        self.text = serge.blocks.utils.addSpriteActorToWorld(world, 'text', 'text', 'help-screen-1', 'foreground',
            center_position=L('text-position'))
        #
        # Back link
        back = serge.blocks.utils.addSpriteActorToWorld(world, 'button', 'back',
            sprite_name='next-button',
            layer_name='foreground',
            center_position=L('help-position'))
        back.linkEvent(serge.events.E_LEFT_CLICK, self.nextClick)

    def nextClick(self, obj, arg):
        """Next was clicked"""
        serge.sound.Sounds.play('click')
        if self.text.getSpriteName() == 'help-screen-1':
            self.text.setSpriteName('help-screen-2')
        else:
            self.engine.setCurrentWorldByName('main-screen')
            self.text.setSpriteName('help-screen-1')

    def updateActor(self, interval, world):
        """Update this actor"""
        super(IntroScreen, self).updateActor(interval, world)
            
    
def main(options):
    """Create the main logic"""
    #
    # The screen actor
    s = IntroScreen(options)
    world = serge.engine.CurrentEngine().getWorld('intro-screen')
    world.addActor(s)
    #
    # The behaviour manager
    manager = serge.blocks.behaviours.BehaviourManager('behaviours', 'behaviours')
    world.addActor(manager)
    manager.assignBehaviour(None, serge.blocks.behaviours.KeyboardQuit(), 'keyboard-quit')
    #
    # Screenshots
    if options.screenshot:
        manager.assignBehaviour(None, 
            serge.blocks.behaviours.SnapshotOnKey(key=pygame.K_s, size=G('screenshot-size')
                , overwrite=False, location='screenshots'), 'screenshots')

