"""The main screen for the game"""

import random
import math
import pygame
import os

import serge.actor
import serge.visual
import serge.events
import serge.common
import serge.blocks.utils
import serge.blocks.visualblocks
import serge.blocks.behaviours
import serge.blocks.actors

if serge.common.PYMUNK_OK:
    import pymunk
    import serge.physical

from theme import G, theme
import serge.sound
import common
import model
import powerupui
import goldui
import serge.blocks.layout


class PowerUpScreen(serge.blocks.actors.ScreenActor):
    """The logic for the main screen"""

    def __init__(self, options):
        """Initialise the screen"""
        super(PowerUpScreen, self).__init__('item', 'powerup-screen')
        self.options = options

    def addedToWorld(self, world):
        """Added to the world"""
        super(PowerUpScreen, self).addedToWorld(world)
        #
        background = serge.blocks.utils.addSpriteActorToWorld(world, 'bg', 'bg', 'power-up-bg', 'background',
            center_position=G('screen-center'))
        #
        self.power_ups = common.POWER_UPS
        #
        self.back_button = serge.blocks.utils.addSpriteActorToWorld(
            world,
            'button', 'back-button',
            'back-button',
            'ui',
            center_position=G('back-button-pos')
        )
        self.back_button.linkEvent(serge.events.E_LEFT_CLICK, self.clickBackButton)
        #
        self.grid = serge.blocks.utils.addActorToWorld(
            world,
            serge.blocks.layout.Grid(
                'grid', 'grid',
                (3, 3),
                G('grid-width'),
                G('grid-height'),
            ),
            center_position=G('grid-posn'),
            layer_name='ui',
        )
        #
        for item in G('power-up-order'):
            items = self.power_ups.classes[item]
            item = items[0]
            new_ui = powerupui.PowerUpUI('power-up', 'power-up', item)
            self.grid.autoAddActor(new_ui)
            new_ui.linkEvent(common.E_POWER_UP_BOUGHT, self.buyPowerUp)

        #
        self.gold_ui = serge.blocks.utils.addActorToWorld(
            world,
            goldui.GoldDisplay('gold', 'gold'),
            center_position=G('gold-posn'),
        )

    def buyPowerUp(self, obj, arg):
        """Buy a power up"""
        serge.sound.Sounds.play('click')
        serge.sound.Sounds.play('collect-gold')
        self.gold_ui.addGold(-obj.cost)
        common.PLAYER.addPowerUp(obj)

    def updateActor(self, interval, world):
        """Update the actor"""
        super(PowerUpScreen, self).updateActor(interval, world)
        #
        gold = self.gold_ui.getGold()
        for actor in self.grid.getChildren():
            actor.setGoldAmount(gold)

    def clickBackButton(self, obj, arg):
        """Clicked the back button"""
        serge.sound.Sounds.play('click')
        #
        # Sync the gold displayed
        screen = self.engine.getWorld('main-screen').findActorByName('main-screen')
        screen.gold_ui.setGold(self.gold_ui.getGold(), self.gold_ui.target_gold)
        screen.updateModelDisplay()
        #
        self.engine.setCurrentWorldByName('main-screen')

    def resetDisplay(self):
        """Reset the display"""
        for actor in self.grid.getChildren():
            actor.updateDisplay()


def main(options):
    """Create the main logic"""
    #
    # The screen actor
    s = PowerUpScreen(options)
    world = serge.engine.CurrentEngine().getWorld('powerup-screen')
    world.addActor(s)
    #
    # The behaviour manager
    manager = serge.blocks.behaviours.BehaviourManager('behaviours', 'behaviours')
    world.addActor(manager)
    manager.assignBehaviour(None, serge.blocks.behaviours.KeyboardBackWorld(), 'keyboard-back')
    #
    # Screenshots
    if options.screenshot:
        manager.assignBehaviour(None,
            serge.blocks.behaviours.SnapshotOnKey(key=pygame.K_s, size=G('screenshot-size')
                , overwrite=False, location='screenshots'), 'screenshots')

