"""The start screen for the game"""

import random
import math
import pygame


import serge.actor
import serge.visual
import serge.sound
import serge.events
import serge.common
import serge.blocks.utils
import serge.blocks.visualblocks
import serge.blocks.behaviours
import serge.blocks.actors

if serge.common.PYMUNK_OK:
    import pymunk
    import serge.physical

from theme import G, theme
import common 
import mainscreen


class StartScreen(serge.blocks.actors.ScreenActor):
    """The logic for the start screen"""

    def __init__(self, options):
        """Initialise the screen"""
        super(StartScreen, self).__init__('item', 'main-screen')
        self.options = options

    def addedToWorld(self, world):
        """The start screen was added to the world"""
        super(StartScreen, self).addedToWorld(world)
        #
        # Logo
        the_theme = theme.getTheme('start-screen')
        L = the_theme.getProperty
        # logo = serge.blocks.utils.addSpriteActorToWorld(world, 'logo', 'logo', 'icon', 'foreground',
        #     center_position=L('logo-position'))
        background = serge.blocks.utils.addSpriteActorToWorld(world, 'bg', 'bg', 'title-page', 'background',
                                                              center_position=L('screen-center'))
        #
        serge.blocks.utils.addTextItemsToWorld(world, [
            # (L('title'), 'title'),
            ('v' + common.version, 'version'),
            ('Start (easy)', 'start-1',  self.startEasyClick),
            ('Start (hard)', 'start-2',  self.startHardClick),
            ('Continue', 'continue',  serge.blocks.utils.worldCallback('main-screen', 'click')),
            ('Help', 'help',  serge.blocks.utils.worldCallback('help-screen', 'click')),
            ('Credits', 'credits',  serge.blocks.utils.worldCallback('credits-screen', 'click')),
            # ('Achievements', 'achievements', serge.blocks.utils.worldCallback('achievements-screen', 'click')),
        ],
                                               the_theme, 'foreground')
        #
        world.linkEvent(serge.events.E_ACTIVATE_WORLD, self.activateWorld)
        self.activateWorld(None, None)

    def activateWorld(self, obj, arg):
        """Activated"""
        if not serge.sound.Music.isPlayingSong('start-music'):
            serge.sound.Music.play('start-music', loops=-1)
            serge.sound.Music.setVolume(1.0)

    def updateActor(self, interval, world):
        """Update this actor"""
        super(StartScreen, self).updateActor(interval, world)

    def startEasyClick(self, obj, arg):
        """Start in easy mode"""
        common.GOLD_MULTIPLIER = 1.5
        self.startClick()

    def startHardClick(self, obj, arg):
        """Start in hard mode"""
        common.GOLD_MULTIPLIER = 1.0
        self.startClick()

    def startClick(self):
        """Restart the entire game"""
        self.engine.removeWorld(self.engine.getWorld('main-screen'))
        serge.blocks.utils.createWorldsForEngine(self.engine, ['main-screen'])
        mainscreen.main(self.options)
        serge.sound.Sounds.play('click')
        self.engine.setCurrentWorldByName('intro-screen')
        mute = serge.blocks.actors.MuteButton('mute-button', 'ui', alpha=G('mute-button-alpha'))
        serge.blocks.utils.addMuteButtonToWorlds(mute, center_position=G('mute-button-position'), world_names=['main-screen'])





            
def main(options):
    """Create the main logic"""
    #
    # The screen actor
    s = StartScreen(options)
    world = serge.engine.CurrentEngine().getWorld('start-screen')
    world.addActor(s)
    #
    # The behaviour manager
    manager = serge.blocks.behaviours.BehaviourManager('behaviours', 'behaviours')
    world.addActor(manager)
    manager.assignBehaviour(None, serge.blocks.behaviours.KeyboardQuit(), 'keyboard-quit')
    #
    # Screenshots
    if options.screenshot:
        manager.assignBehaviour(None, 
            serge.blocks.behaviours.SnapshotOnKey(key=pygame.K_s, size=G('screenshot-size')
                , overwrite=False, location='screenshots'), 'screenshots')

