"""Display waves"""

import random

import serge.common
import serge.blocks.actors
import serge.blocks.layout
import serge.blocks.visualblocks
import serge.actor
from theme import G, theme



class WavesDisplay(serge.actor.MountableActor):
    """Display the waves"""

    def __init__(self, tag, name, progress_name):
        """Initialise"""
        super(WavesDisplay, self).__init__(tag, name)
        #
        self.progress_name = progress_name

    def addedToWorld(self, world):
        """Added to the world"""
        super(WavesDisplay, self).addedToWorld(world)
        #
        # Progress Display
        self.progress_display = serge.actor.Actor('%s hp' % self.tag, 'bar')
        self.progress_display.visual = serge.blocks.visualblocks.ProgressBar(
            G('%s-progress-size' % self.name),
            value_ranges=[(0.0, 10.0, G('%s-progress-colour' % self.name))],
        )
        self.mountActor(self.progress_display, G('%s-progress-posn' % self.name))
        self.progress_display.setLayerName('ui-front')
        self.progress_display.visual.value = 10.0
        #
        # Text display
        self.progress_text = serge.blocks.actors.FormattedText(
            'progress-text', 'progress-text',
            format='%(progress_name)s %(progress)d / %(progress_max)d',
            colour=G('%s-progress-colour' % self.name),
            font_size=G('%s-progress-font-size' % self.name),
            justify='left',
            progress=10,
            progress_max=10,
            progress_name=self.progress_name,
        )
        self.progress_text.setLayerName('ui-front')
        self.mountActor(self.progress_text, G('%s-progress-text-posn' % self.name))
        
    def setProgress(self, progress):
        """Set the current progress"""
        self.progress_text.setValue('progress', progress)

    def updateActor(self, interval, world):
        """Update the actor"""
        self.progress_display.visual.value = 0.9 * self.progress_display.visual.value + 0.1 * \
                                                self.progress_text.getValue('progress')
