"""Simulate the running of a battle"""

import os
import random
import collections
from game import model



loader = model.EnemyLoader(os.path.join('..', 'game', 'enemies.csv'))
powerups = model.PowerUpLoader(os.path.join('..', 'game', 'powerups.csv'))


def report(results):
    """Report results"""
    for result in results:
        if result:
            print result.getReportString()


def simulate(wave, number, do_report=False, max_iterations=1000, buy_stuff=True):
    enemies = loader.getEnemyGroup(wave, number)
    powerups.resetPowerUps()
    p = model.Player()
    p.auto_click_interval = 0.1

    for i in range(max_iterations):
        #
        # Player attacks
        results = p.tickClock(0.05, enemies)
        if do_report:
            report(results)
        #
        # Get gold
        new_gold = 0
        for result in results:
            new_gold += result.value
        if new_gold:
            p.addMoney(new_gold)
            if do_report:
                print 'Add gold %s, %s' % (new_gold, str(p))
        #
        # Buy stuff
        if buy_stuff:
            items = powerups.getItemsAvailable(p.purse)
            if items:
                item = random.choice(items)
                p.addPowerUp(item)
                if do_report:
                    print 'Bought %s (%s)' % (item, str(p))
        #
        # Enemies attack
        results = enemies.tickClock(0.05, [p])
        if do_report:
            report(results)

        #
        if not p.is_alive:
            return 'Enemies', i, (p, e)

        if not [e for e in enemies if e.is_alive]:
            return 'Player', i, (p, e)



def tournament(wave, number, iterations):
    """Perform a tournament"""
    results = collections.OrderedDict()
    results['Player'] = 0
    results['Enemies'] = 0
    results['Sidekick-A active'] = 0
    results['Sidekick-B active'] = 0
    results['Sidekick-C active'] = 0
    results['Spent'] = 0
    results['Power ups'] = 0
    results['End gold'] = 0
    #
    # Winners and losers
    for i in range(iterations):
        winner, rounds, (player, enemies) = simulate(wave, number)
        results[winner] += 1
        #
        # Sidekicks active
        for name, sidekick in player.sidekicks.iteritems():
            if sidekick.is_alive:
                results['%s active' % name] += 1.0 / iterations * 100.0
        #
        # Power ups
        results['Spent'] += float(player.total_spent) / iterations
        results['Power ups'] += float(player.total_power_ups) / iterations
        results['End gold'] += float(player.purse) / iterations
        #
    for key, value in results.iteritems():
        print key, float(value) / iterations * 100.0


won, rounds, (player, enemies) = simulate(1, 3, True)